import {
  decodeIterable,
  encodeIterable,
  decodeCallback
} from 'ipfs-message-port-protocol/core';
import {
  decodeCID,
  encodeCID
} from 'ipfs-message-port-protocol/cid';
export class CoreService {
  constructor(ipfs) {
    this.ipfs = ipfs;
  }
  addAll(query) {
    const {input} = query;
    const {chunker, cidVersion, enableShardingExperiment, hashAlg, onlyHash, pin, progressCallback, rawLeaves, shardSplitThreshold, trickle, wrapWithDirectory, timeout, signal} = query;
    let progress;
    if (progressCallback) {
      const fn = decodeCallback(progressCallback);
      progress = (bytes, fileName) => {
        fn([
          bytes,
          fileName
        ]);
      };
    }
    const options = {
      chunker,
      cidVersion,
      enableShardingExperiment,
      hashAlg,
      onlyHash,
      pin,
      rawLeaves,
      shardSplitThreshold,
      trickle,
      wrapWithDirectory,
      timeout,
      progress,
      signal
    };
    const content = decodeAddAllInput(input);
    return encodeAddAllResult(this.ipfs.addAll(content, options));
  }
  async add(query) {
    const {input} = query;
    const {chunker, cidVersion, hashAlg, onlyHash, pin, progressCallback, rawLeaves, trickle, wrapWithDirectory, timeout, signal} = query;
    let progress;
    if (progressCallback) {
      const fn = decodeCallback(progressCallback);
      progress = (bytes, fileName) => {
        fn([
          bytes,
          fileName
        ]);
      };
    }
    const options = {
      chunker,
      cidVersion,
      hashAlg,
      onlyHash,
      pin,
      rawLeaves,
      trickle,
      wrapWithDirectory,
      timeout,
      progress,
      signal
    };
    const content = decodeAddInput(input);
    return encodeAddResult(await this.ipfs.add(content, options));
  }
  cat(query) {
    const {path, offset, length, timeout, signal} = query;
    const location = typeof path === 'string' ? path : decodeCID(path);
    const content = this.ipfs.cat(location, {
      offset,
      length,
      timeout,
      signal
    });
    return encodeCatResult(content);
  }
  ls(query) {
    const {path, recursive, preload, timeout, signal} = query;
    const location = typeof path === 'string' ? path : decodeCID(path);
    const entries = this.ipfs.ls(location, {
      recursive,
      preload,
      timeout,
      signal
    });
    return encodeLsResult(entries);
  }
}
const decodeAddAllInput = input => decodeIterable(input, decodeFileInput);
const decodeAddInput = input => matchInput(input, data => {
  if (data.type === 'RemoteIterable') {
    return { content: decodeIterable(data, decodeFileInput) };
  } else {
    return decodeFileInput(data);
  }
});
const decodeFileInput = input => matchInput(input, file => ({
  ...file,
  content: file.content && decodeFileContent(file.content)
}));
const decodeFileContent = content => matchInput(content, input => decodeIterable(input, identity));
const matchInput = (input, decode) => {
  if (typeof input === 'string' || input instanceof ArrayBuffer || input instanceof Blob || ArrayBuffer.isView(input)) {
    return input;
  } else {
    return decode(input);
  }
};
const encodeAddAllResult = out => {
  const transfer = new Set();
  return {
    data: encodeIterable(out, encodeFileOutput, transfer),
    transfer
  };
};
const encodeAddResult = out => {
  const transfer = new Set();
  return {
    data: encodeFileOutput(out, transfer),
    transfer
  };
};
const encodeCatResult = content => {
  const transfer = new Set();
  return {
    data: encodeIterable(content, moveBuffer, transfer),
    transfer
  };
};
const encodeLsResult = entries => {
  const transfer = new Set();
  return {
    data: encodeIterable(entries, encodeLsEntry, transfer),
    transfer
  };
};
const encodeLsEntry = ({name, path, size, cid, type, mode, mtime}) => ({
  cid: encodeCID(cid),
  type,
  name,
  path,
  mode,
  mtime,
  size
});
const moveBuffer = (buffer, transfer) => {
  transfer.add(buffer.buffer);
  return buffer;
};
const encodeFileOutput = (file, _transfer) => ({
  ...file,
  cid: encodeCID(file.cid)
});
const identity = v => v;